#ifndef __UTILS_H__
#define __UTILS_H__

#include <stdlib.h>

#include "log.h"

#ifndef uint
#define uint                                                    unsigned int
#endif

#define ftl(...)                                                {__ftl(__VA_ARGS__); abort();}

#define abort_if_null(x)                                        if(x == NULL) {ftl(#x " cannot be null");}

#define ABS(X)                                                  ((X) < 0 ? (-X) : (X))
#define SWAP(X, Y, T)                                           do { T s = X; X = Y; Y = s; } while(0)
#define UNUSED(x)                                               ((void)(x))
#define CLEAR(x)                                                memset((void *)(x), 0, sizeof(*(x)))

#define DYNAMIC_ARRAY(NAME, T)                                  \
    typedef struct NAME {                                       \
        T *data;                                                \
        size_t used;                                            \
        size_t reserved;                                        \
    } NAME;                                                     \
    static inline void init_##NAME(NAME *v) {                   \
        v->data = NULL; v->used = v->reserved = 0;              \
    }                                                           \
    static inline void reserve_##NAME(NAME *v, size_t n) {      \
        v->reserved += n;                                       \
        v->data = (T *)realloc(v->data, v->reserved*sizeof(T)); \
    }                                                           \
    static inline void shrink_##NAME(NAME *v) {                 \
        v->reserved = v->used;                                  \
        v->data = (T *)realloc(v->data, v->reserved*sizeof(T)); \
    }                                                           \
    static inline void append_##NAME(NAME *v, T *t) {           \
        if(v->reserved <= v->used)                              \
            reserve_##NAME(v, 1);                               \
        v->data[v->used++] = *t;                                \
    }                                                           \
    static inline void free_##NAME(NAME *v) {                   \
        free(v->data);                                          \
        memset(v, 0, sizeof(T));                                \
    }

#endif
